/*
 * DS18b20.cpp
 *
 * Created: 27.09.2020 3:24:41
 *  Author: 123
 */ 

#include "DS18b20.h"

DS18B20::DS18B20 (unsigned char pin) {
	this -> DQ = pin;
}

unsigned char DS18B20::reset( void ) {
	int pin = this -> DQ;
    pinMode(pin, 1);
    _delay_us(520);
    MOW_CLI();
    pinMode(pin, 0);
    _delay_us(2);
    for (uint8_t c = 255; c; c--) {
	    if (!digitalRead(pin)) {
		    MOW_SEI();
		    for (uint8_t i = 255; !digitalRead(pin) && i; i--) _delay_us(1);
		    return false;
	    }
	    _delay_us(1);
    }
    MOW_SEI();
    return true;
}

unsigned char DS18B20::read_packet( void ) {
    uint8_t data = 0;						 
    for (uint8_t p = 8; p; p--) {
	    data >>= 1;
	    MOW_CLI();
	    pinMode(this -> DQ, 1);
	    _delay_us(2);
	    pinMode(this -> DQ, 0);
	    _delay_us(8);
	    if (digitalRead(this -> DQ)) data |= 1 << 7;
	    _delay_us(60);
	    MOW_SEI();
    }
    return data;
}

void DS18B20::write_packet( unsigned char byte ) {
    for (uint8_t p = 8; p; p--) {
	    pinMode(this -> DQ, 1);
	    MOW_CLI();
	    if (byte & 1) {
		    _delay_us(5);
		    pinMode(this -> DQ, 0);
		    _delay_us(60);
		} else {
		    _delay_us(60);
		    pinMode(this -> DQ, 0);
		    _delay_us(5);
	    }
	    MOW_SEI();
	    byte >>= 1;
    }
}

void DS18B20::beginConversion( void ) {
	this -> reset();
	this -> write_packet(CMD_SKIPROM);
	this -> write_packet(CMD_CONVERTTEMP);
}

double DS18B20::readTemp( void ) {
	unsigned char tempL;
	unsigned char tempH;
	
	this -> reset();
	this -> write_packet(CMD_SKIPROM);
	this -> write_packet(CMD_RSCRATCHPAD);
	
	tempL = this -> read_packet();
	tempH = this -> read_packet();
	
	return (double)((tempH << 8) + tempL) * 0.0625D;
}